/*!
 * gameMain.js
 * author: Brent M. Dingle
 * last modified: 2015
 * Example game script for overlay of multiple canvas elements
 * The first canvas is basically a title/menu screen
 * The second is a 'snow game' page
 *
 * All canvas elements are to be resized with the screen
 */

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// 'class' menuButton											menuButton
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
function menuButton()
{
	this.boxColor          = "rgb(200, 200, 200)";
	this.boxShadowColor    = "rgb(100, 100, 100)";
	this.x                 = 20;
	this.y                 = 20;
	this.width             = 240;
	this.height            = 40;
	this.text              = "Button Text";
	this.textColor         = "rgb(0, 0, 256)";
}

// ----------------------------------------------------------------------------
// menuButton 'class' method
// ----------------------------------------------------------------------------
menuButton.prototype.coordsAreInside = function(mouseX, mouseY)
{
    if (   ((mouseX > this.x) && (mouseX < (this.x + this.width))) 
		&& ((mouseY > this.y) && (mouseY < (this.y + this.height))) )
		return true;
	else
		return false;	
}

 
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// theGame Object											theGame Object
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
var theGame = 
{
	// ----------------------------------------------------------------------------
	// Pseudo-constants
	// ----------------------------------------------------------------------------
	STATE_MENU:    0,
	STATE_CREDITS: 1,
	STATE_SNOW:    5,
	
	// ----------------------------------------------------------------------------
	// Variables
	// ----------------------------------------------------------------------------
	gameState: 0,       // originally: 0 = menu, 1 = credits, 5 = snowGame
	isInitialized: 0,   // 0 = false, indicates if Init() function has been called

	angle: 0,           // for particles
	mp: 35,             // max particles
	particles: [],      // snow particles
	
	menuBckgrndImg: new Image(),
	menuBckgrndLoaded: 0,
	menuB1: new menuButton(),

	// ----------------------------------------------------------------------------
	// ----------------------------------------------------------------------------
	Init: function ()
	{
		this.menuBckgrndImg.src = 'bckgrnd.png';
		this.menuBckgrndImg.onload = function () { 	theGame.menuBckgrndLoaded=1; };
													//console.log("DEBUG: menu background loaded!"); };

		
		// Init Local Canvas variables
		var menuCanvas = document.getElementById("canvasMenu");
		var snowCanvas = document.getElementById("canvasSnow");

		// ----------------------------------- Setup Canvases to register EVENTS
		// See also: http://www.w3schools.com/tags/ref_eventattributes.asp
		// for a complete list of events in HTML5
		// Aside: Keyboard events may be better 'caught' by the window rather than the canvas
		menuCanvas.addEventListener('click', theGame.doMenuClick, false);

		snowCanvas.addEventListener('click', 
							function() { snowCanvas.style.display="none";
										  menuCanvas.style.display="block";
										  theGame.gameState = theGame.STATE_MENU;
										}, false);
		
		// The above use eventListeners, but you can also do the following, if functions are 'short'
		// snowCanvas.onclick = function() 	{	snowCanvas.style.display="none";
		//								  		menuCanvas.style.display="block";
		//								  		theGame.gameState = theGame.STATE_MENU;
		//									};
		
		//canvas dimensions -- if allowing window resize and center
		var W = snowCanvas.width;
		var H = snowCanvas.height;

		//snowflake particles
		//var particles = [];
		for(var i = 0; i < this.mp; i++)
		{
			this.particles.push({
				x: Math.random()*W, //x-coordinate
				y: Math.random()*H, //y-coordinate
				r: Math.random()*4+1, //radius
				d: Math.random()*this.mp //density
			})
		}
		
		// hide the snowCanvas
		snowCanvas.style.display="none";
		
		this.isInitialized = 1;  // 0 is false
		this.gameState = this.STATE_MENU;
	},
	
	getMousePos: function (canvas, evt)
	{
		var rect = canvas.getBoundingClientRect();
		return 	{
					// add Math.round( ) around the below to get rid of excessive decimals
					x: Math.round( (evt.clientX-rect.left)/(rect.right-rect.left)*canvas.width ),
					y: Math.round( (evt.clientY-rect.top)/(rect.bottom-rect.top)*canvas.height )
				};
	},
	
	// ----------------------------------------------------------------------------
	// ----------------------------------------------------------------------------
	doMenuClick: function (evt)
	{
		var menuCanvas = document.getElementById("canvasMenu");
		var snowCanvas = document.getElementById("canvasSnow");

		var mousePos = theGame.getMousePos(menuCanvas, evt);

		//console.log("DEBUG: doMenuClick, mouse pos = " + mousePos.x + ", " + mousePos.y);
		//console.log("DEBUG: doMenuClick, return val of coordsAreInside = " + theGame.menuB1.coordsAreInside(mousePos.x, mousePos.y));
		
		if ( theGame.menuB1.coordsAreInside(mousePos.x, mousePos.y) == true)
		{
			snowCanvas.style.display="block";
			menuCanvas.style.display="none";
			theGame.gameState = theGame.STATE_SNOW;
		}
	},

	// ----------------------------------------------------------------------------
	// UpdateSnow
	// Function to move the snowflakes
	// angle will be an ongoing incremental flag. 
	// sin and cos functions will be applied to it 
	// to create vertical and horizontal movements of the flakes
	// ----------------------------------------------------------------------------
	UpdateSnow: function ()
	{
		//console.log("DEBUG: UpdateSnow called");
		
		// Init Local Canvas variables
		var snowCanvas = document.getElementById("canvasSnow");
		var gameCTX = snowCanvas.getContext("2d");

		//canvas dimensions -- if allowing window resize and center
		var W = snowCanvas.width;
		var H = snowCanvas.height;
		
		//console.log("DEBUG: mp = " + this.mp);
		
		this.angle += 0.01;
		for(var i = 0; i < this.mp; i++)
		{
			var p = this.particles[i];
			
			//console.log("p.x = " + p.x);
			
			//Updating X and Y coordinates
			//We will add 1 to the cos function to prevent negative values which will lead flakes to move upwards
			//Every particle has its own density which can be used to make the downward movement different for each flake
			//Lets make it more random by adding in the radius
			p.y += Math.cos(this.angle+p.d) + 1 + p.r/2;
			p.x += Math.sin(this.angle) * 2;
			
			//Sending flakes back from the top when it exits
			//Lets make it a bit more organic and let flakes enter from the left and right also.
			if(p.x > W+5 || p.x < -5 || p.y > H)
			{
				if(i%3 > 0) //66.67% of the flakes
				{
					this.particles[i] = {x: Math.random()*W, y: -10, r: p.r, d: p.d};
				}
				else
				{
					//If the flake is exitting from the right
					if(Math.sin(this.angle) > 0)
					{
						//Enter from the left
						this.particles[i] = {x: -5, y: Math.random()*H, r: p.r, d: p.d};
					}
					else
					{
						//Enter from the right
						this.particles[i] = {x: W+5, y: Math.random()*H, r: p.r, d: p.d};
					}
				}
			}
		}
	},

	// ----------------------------------------------------------------------------
	// Draw the SnowFlakes
	// ----------------------------------------------------------------------------
	DrawSnow: function ()
	{
		//console.log("DEBUG: DrawSnow called");
		
		// Init Local Canvas variables
		var snowCanvas = document.getElementById("canvasSnow");
		var gameCTX = snowCanvas.getContext("2d");

		// -Draw the Game of 'snow'
		// Game canvas dimensions -- if allowing window resize and center
		var W = snowCanvas.width;
		var H = snowCanvas.height;
		
		gameCTX.clearRect(0, 0, W, H);
		
		gameCTX.fillStyle = "rgba(255, 255, 255, 0.8)";
		gameCTX.beginPath();
		for(var i = 0; i < this.mp; i++)
		{
			var p = this.particles[i];
			gameCTX.moveTo(p.x, p.y);
			gameCTX.arc(p.x, p.y, p.r, 0, Math.PI*2, true);
		}
		gameCTX.fill();
		
	},

	// ----------------------------------------------------------------------------
	// Draw the Menu Screen
	// ----------------------------------------------------------------------------
	DrawMenu: function ()
	{
		// Init Local Canvas variables
		var menuCanvas = document.getElementById("canvasMenu");
		var menuContext = menuCanvas.getContext("2d");

		// ------- Simple Menu for Testing/Loading
		menuContext.font="30px Verdana";
		// Create gradient
		var gradient=menuContext.createLinearGradient(0,0,menuCanvas.width,0);
		gradient.addColorStop("0","magenta");
		gradient.addColorStop("0.5","blue");
		gradient.addColorStop("1.0","red");
		// Fill with gradient
		menuContext.fillStyle=gradient;
		menuContext.fillText("Loading",10,50);


		//console.log("DEBUG: menuBckgrndLoaded = " + this.menuBckgrndLoaded);
		if (this.menuBckgrndLoaded != 0) // && (this.menuB1loaded != 0) && (this.menuB2loaded != 0) ...
		{
			// draw background image
			menuContext.drawImage(this.menuBckgrndImg, 0, 0);
			
			// draw buttons (not using images here)
			// menuContext.fillStyle = "rgb(0, 0, 128)";  // buttonObj.rgb;
			// menuContext.fillRect(20, 20, 120, 40);     // x, y, width, height
			
			menuContext.fillStyle = this.menuB1.boxShadowColor;
			menuContext.fillRect(this.menuB1.x+2, this.menuB1.y+2, this.menuB1.width, this.menuB1.height);
			menuContext.fillStyle = this.menuB1.boxColor;
			menuContext.fillRect(this.menuB1.x, this.menuB1.y, this.menuB1.width, this.menuB1.height);
			menuContext.fillStyle = this.menuB1.textColor;
			menuContext.fillText("Snow Example",this.menuB1.x+5, this.menuB1.y + this.menuB1.height - 10);
		}
		
	},		

	// ----------------------------------------------------------------------------
	// Update Based on this.gameState
	// ----------------------------------------------------------------------------
	Update: function ()
	{
		//console.log("DEBUG: Update called, gameState = " + this.gameState );
		
		if (this.gameState == this.STATE_SNOW)
		{
			theGame.UpdateSnow();
		}
		// menu and credits screen do not need updates
		// however Init() should have setup some onClick callbacks
		// that will change game state
		else if ((this.gameState != this.STATE_MENU) && (this.gameState != this.STATE_CREDITS))
		{
			console.log("ERROR: Update() -- unknown gameState = " + this.gameState );
		}
	},
	
	// ----------------------------------------------------------------------------
	// Draw Based on this.gameState
	// Note: Draw should be called BEFORE update in runGame()
	// ----------------------------------------------------------------------------
	Draw: function ()
	{
		//console.log("DEBUG: Draw called, gameState = " + this.gameState );
		
		if (this.gameState == this.STATE_SNOW)
		{
			theGame.DrawSnow();
		}
		else if (this.gameState == this.STATE_MENU)
		{
			theGame.DrawMenu();
		}
		else
		{
			console.log("ERROR: Draw() -- unknown gameState = " + this.gameState );
		}
	},

	
};   // end of theGame variable/object
	

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// Window Handling Stuff
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
window.addEventListener("resize", doResize, false);
window.addEventListener("keydown", doKeyDown, false);

// ----------------------------------------------------------------------------
//                                                             doResize
// ----------------------------------------------------------------------------
function doResize() {

	var canvas1 = document.getElementById("canvasMenu");
	var canvas2 = document.getElementById("canvasCredits");
	var canvas3 = document.getElementById("canvasSnow");
	
	// Size things
	UpdateCanvas(canvas1);
	UpdateCanvas(canvas2);
	UpdateCanvas(canvas3);

	// If game has been initialized then force a game redraw
	// Note the game itself is NOT updated here... just changed the size of view
	if (theGame.isInitialized != 0)
	{
		theGame.Draw();
	}
};

// ----------------------------------------------------------------------------
//                                                               UpdateCanvas
// ----------------------------------------------------------------------------
function UpdateCanvas(canvas) {
	
	//console.log("UpdateCanvas Called");
	
	// Here's the tricky part 
	// - Set the width and height of the canvas's coordinate system
	// - if not specified it defaults to width=300 and height=150
	// This is the size you "pretend" your world is (i.e. optimal)
	// 
	canvas.width = 800;
	canvas.height = 480;
	
	// DEBUG: console.log(canvas.width + " " + canvas.height);
	// DEBUG: console.log("W " + window.innerWidth + " " + window.innerHeight);
	
	// Now we consider the actual display (the style width and height)
	// -- this is the size of the window you are using to view your world

    // canvas.style.width = window.innerWidth + 'px';       // This line and the next resize things
    // canvas.style.height = window.innerHeight + 'px';     // but do not maintain scale ratio
	
	// The below scale the game and maintain a ratio of 800x480 
	// by "adding" empty space to the right and bottom of the canvas as appropriate
	// You may need to put a box around the canvas to better see this
	
	var gameWidth = window.innerWidth;
	var gameHeight = window.innerHeight;
	var scaleToFitX = gameWidth / 800;
	var scaleToFitY = gameHeight / 480;

	var currentScreenRatio = gameWidth / gameHeight;
	var optimalRatio = Math.min(scaleToFitX, scaleToFitY);

	canvas.style.position = "fixed";
	
	if (currentScreenRatio >= 1.77 && currentScreenRatio <= 1.79) {
		canvas.style.width = gameWidth + "px";
		canvas.style.height = gameHeight + "px";
		canvas.style.left = 0 + "px";
	}
	else {
		canvas.style.width = 800 * optimalRatio + "px";
		canvas.style.height = 480 * optimalRatio + "px";
		canvas.style.left = ( (gameWidth - (800 * optimalRatio)) / 2 ) + "px";

	}
};

// ----------------------------------------------------------------------------
//                                                                doKeyDown
//
// Setup things here so ESCAPE key will always return viewer to
// the menu screen.
// May also handle user keyboard input here (note canvas is not full screen)
// ----------------------------------------------------------------------------
function doKeyDown(e)
{ 
	//alert(e.keyCode)       // for debugging

	if (e.keyCode == 27)
	{
		var menuCanvas = document.getElementById("canvasMenu");
		var snowCanvas = document.getElementById("canvasSnow");
		snowCanvas.style.display="none";
		menuCanvas.style.display="block";
		theGame.gameState = theGame.STATE_MENU;
	}
	
	// Then likely would do 
	// theGame.processKeyPress(e.keydown);
	
	// Depending on the game may need a keylistener for keyup and keypress also
};
 

// ----------------------------------------------------------------------------
//                                                        window.ONLOAD
// ----------------------------------------------------------------------------
window.onload = function(){

	// Size all canvas to correct Ratio before doing anything 
	// Needed for resizing and centring to work correctly from the start
	doResize();
	
	// Initialize the game
	theGame.Init();

	// Run the game (start the game loop)
	runGame();
};

// ----------------------------------------------------------------------------
//                                                         the game 'loop'
// Event handlers/listeners cover the game input
// ----------------------------------------------------------------------------
function runGame()
{
	// This 'game' code is setup to draw THEN update
	theGame.Draw();             
	theGame.Update();

    // Be certain to call runGame again	
	requestAnimationFrame(runGame);
};
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// end gameMain.js
